/** \file 
 *
 * dabusb.c -- Startup and main dispatcher for USB firmware
 *
 *  $Id: dabusb.c,v 1.14.2.15 2003/06/23 15:54:22 acher Exp $
 */

#include "regezusb.h"

#pragma SAVE
#pragma NOOVERLAY

/* Memory map (preliminary):

   0000-1fff  EZ-USB code/data ram
   2000-27ff  EZ-USB isochronous fifos
   2800-7b3f  reserved for future (FPGA mirrored)
   7b40-7fff  EZ-USB bulk buffer, HW registers
   8000-83ff  FPGA reserved     [a(11:10)=00]
   8400-87ff  D-FIRE DP-RAM CS  [a(11:10)=01]
   8800-8fff  FPGA reserved     [a(11:10)=1x]
   9000-ffff  reserved (FPGA mirrored)
*/ 

#ifdef KEY12
#define VENDOR_STR  "TerraTec        "
#else
#define VENDOR_STR  "BayCom          "
#endif

#ifdef KEY12
#define VERSION_STR "DR-Box 1 V1.13PC"
#else
#define VERSION_STR "DABUSB V1.134PC "
#endif


typedef unsigned char uchar;

#include "io.h"
#include "variables.h"

extern void read_module_buf(void);
extern void write_module_buf(void);
extern void download_fpga(void);
void set_i2s_ep(void);
void set_rdi_ep(void);
void handle_set_feature(void);
void handle_get_feature(void);
extern void write_i2c(void);
extern void read_i2c(void);
extern void read_ee(void);
void select_subch(uchar n);
void select_prog(uchar n,uchar m);
void set_name(void);
void show_name(void);
void init_control(void);
uchar read_nvram(void);
uchar write_nvram(void);
void clear_lcd(void);
void dac_startup(void);
// Control Selectors (CS)

#define SAMPLING_FREQ_CONTROL 0x01
#define PITCH_CONTROL 0x02

#define MUTE_CONTROL 0x01
#define VOLUME_CONTROL 0x02

#define SET_CUR 0x01
#define GET_CUR 0x81

#define SET_MIN 0x02
#define GET_MIN 0x82

#define SET_MAX 0x03
#define GET_MAX 0x83

#define SET_RES 0x04
#define GET_RES 0x84

#define SET_MEM 0x05
#define GET_MEM 0x85

//#define LCD_CONT 0xf
/*-------------------------------------------------------------------*/
// STATIC VARIABLES in internal RAM

char ep2_in_req;
char ep2_out_req;
uchar simulate,last_key,subch,xkey;
extern uchar key,dline;

uchar lcd_cont;
extern  uchar alt_set[4];
uchar ep0_dispatch,ep0o_dispatch;
uchar  xvolume,show_vol, last_locked_key,num_sub,num_sub_temp;

uchar nvram_bank,nvram_addr,nvram_val, fpga_loaded, blink_toggle;

// Variables in external RAM

 unsigned int monpos;
 unsigned int outpos;
 char counter,prog_set, timer;

xdata /*at 0x1b10*/ uchar saved_setup[8];
xdata /*at 0x1b18*/ uchar saved_out0buf[8];
code char* code version_str=VERSION_STR;
code char* code vendor_str=VENDOR_STR;

//xdata at 0x1800 uchar monbuf[512];


/*-------------------------------------------------------------------*/
// IRQ macros
#define IRQ_PRE     _asm \
                    push _DPS \
                    _endasm; \
                    DPS=0;             

#define IRQ_PREUSB  _asm \
                    push _DPS \
                    _endasm; \
                    DPS=0; \
                    _asm \
	            mov a,_EXIF \
                    clr acc.4 \
                    mov _EXIF,a \
                   _endasm;


#define IRQ_POST    _asm \
	            pop _DPS \
                    _endasm;
/*-------------------------------------------------------------------*/
//! waits 0.1ms by cycle counting
void wait_0_1ms(void)
{
        DPS = 0;
/*   Delay for .1 millisecond (100 microseconds). */
/*   10 cycles * 166.6 ns per cycle is 1.66 microseconds per loop. */
/*   100 microseconds / 1.66 = 60.  [assumes 24 MHz clock] */
               
        _asm;
	push dph;
	push dpl;
        mov     dptr,#(0xffff - 60)    ; long pulse for operating
        mov     r4,#5
    loop:
        inc     dptr            ; 3 cycles
        mov     a,dpl           ; 2 cycles
        orl     a,dph           ; 2 cycles
        jnz     loop            ; 3 cycles
        pop dpl
        pop dph					  
        _endasm;
}
/*-------------------------------------------------------------------*/
//! SOF interrupt
/*! reset Xilinx SOF-avoidance-window, scan keyboard
*/
void usb_sof_isr(void) interrupt 14
{
	int n;
	volatile uchar x;
	IRQ_PREUSB;
	x=xilinx0;      // Reset subframe counter
	USBIRQ = 0x02;
	USBIRQ = 0x02;  // errata

	timer++;
	
	if ((timer&63)==0)
		get_key();

#if 1
	if (simulate)
	{
		for(n=0;n<192/8;n++)
		{
			_asm
	    
				mov dptr,#_IN9DATA;
			movx @dptr,a;
			movx @dptr,a;
			movx @dptr,a;
			movx @dptr,a;
			movx @dptr,a;
			movx @dptr,a;
			movx @dptr,a;
			movx @dptr,a;
			_endasm;
		}
      
		// OUTA ^= 0x80;
	}
#endif
	
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! Only used for debugging
void usb_sutok_isr(void) interrupt 15  
{
	IRQ_PREUSB;
	USBIRQ = 0x04;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! USB-suspend interrupt 
void usb_suspend_isr(void) interrupt 16
{
	IRQ_PREUSB;
	USBIRQ = 0x08;
	poscursor(0,2);
	printf("SUPEND");
	USBCS = 0;
	EICON=0x20;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! USB -reset interrupt 
void usb_usbreset_isr(void) interrupt 17
{
	IRQ_PREUSB;
	USBIRQ = 0x10;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! USB resume interupt
void resume_isr(void) interrupt 7
{
	IRQ_PRE;
//	poscursor(0,2);
//	printf("RESUME");
	USBCS = 2;  /* 8051 handles control */
	USBCS = 6;  /* reconnect, 8051 handles control */
	EICON=0x10;
	
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! Mute DAC
void mute_dac(void)
{
	i2c_out3(0x02,0,0);
	i2c_out2(0x03,0x40);
}
/*-------------------------------------------------------------------*/
//! Intialize DAC
void init_dac()
{ 
	xilinx0=0x40 | (lcd_cont&(~DAC_RESET)); //DAC-RESET

	lcd_cont|=0x10;
	xilinx0=0x40 | lcd_cont; //DAC-RESET off, mute on
	wait_0_1ms();
	wait_0_1ms();
	i2c_out2(0x01,0x18   );
	i2c_out3(0x02,0x00,0x00);
	wait_0_1ms();
	i2c_out2(0x03,0x44);

}
/*-------------------------------------------------------------------*/
//! EP 1 OUT interrupt: Handle incoming 
void usb_ep1out_isr(void) interrupt 21
{
	IRQ_PREUSB;
	OUT07IRQ = 0x02;
	
	//ep1_out_req=1;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! EP 2 IN interrupt: Handle usb->host data
void usb_ep2in_isr(void) interrupt 22
{
	IRQ_PREUSB;
	IN07IRQ = 0x04;
	
	ep2_in_req=1;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! EP 2 OUT interrupt: Handle incoming 
void usb_ep2out_isr(void) interrupt 23
{
	IRQ_PREUSB;
	OUT07IRQ = 0x04;
	
	ep2_out_req=1;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>> USB INT END <<<<<<<<<<<<<<<<<<<<<<<< */

/*-------------------------------------------------------------------*/
//! I2C Interrupt

void i2c_isr(void) interrupt 9
{
  IRQ_PRE;
  /* clear interrupt */

  _asm
    mov a,_EXIF
    clr acc.5
    mov _EXIF,a
    _endasm; 

  IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! I2S INT (no longer used)
extern void int4_isr(void) interrupt 10;
 
/*-------------------------------------------------------------------*/
//! DFIRE interrupt (not used)

void int5_isr(void) interrupt 11
{
        IRQ_PRE;
        /* clear interrupt */
        _asm
        mov a,_EXIF
        clr acc.7
        mov _EXIF,a
        _endasm;

	//	msg_wait=1;

	IRQ_POST;
}

/*-------------------------------------------------------------------*/
//! RDI INT 
extern void int6_isr(void) interrupt 12;

/*-------------------------------------------------------------------*/
//! Set alternate setting for Audio interface
void set_i2s_ep(void)
{
 
	alt_set[0]=SETUPDAT[2];
	OUTA = 0x0;
	// enable i2s 
}
/*-------------------------------------------------------------------*/
//! Set alternate setting for RDI interface
void set_rdi_ep(void)
{
	alt_set[1]=SETUPDAT[2];
	// enable rdi
}
/*-------------------------------------------------------------------*/
//! set volume, mapping USB-values -> DAC
void set_volume(void)
{
	volatile uchar cs,cn; 

	cs=volume[19];
	cn=volume[21];

	cs+=0x80;
	cn+=0x80;
	cs=(cs>>2)+(cs>>3);
	cn=(cn>>2)+(cn>>3);

	if (volume[10])
		cs=0;

	if (volume[12])
	{
		cn=0;	
	}

	xvolume=(cn+cs)/2;
	i2c_out3(0x02,cn,cs);
#if 0
	clr_line1();
	printh(volume[10]);
	printh(volume[12]);
	printh(volume[19]);
	printh(volume[21]);
#endif

	if (show_vol==1)
	{
		clr_line1();
		putc('V');
		printi(xvolume,2);
		bargraph(1+60/4,4,3);
		bargraph(1+xvolume/4,4,1);
	}
	
}
/*-------------------------------------------------------------------*/
// cs=1: Mute
// cs=2: VOL
//! handling of USB-audio requests
void handle_set_feature(void)
{
  int len;
  uchar cs,cn,fuid,ifep;

//	poscursor(0,0);
  //  printf("SET FEATURE\n");
  // 
  OUTA ^= 0x80;
  cs=SETUPDAT[3];
  cn=SETUPDAT[2];
  fuid=SETUPDAT[5];
  ifep=SETUPDAT[4];
  len=SETUPDAT[6]+((int)SETUPDAT[7]<<8);
//  printh(cs);
//  printh(cn);
  if ((SETUPDAT[1]==SET_CUR))
    {
      if (len==1)
	{
	  volume[cs*8+2*cn]=OUT0BUF[0];
	}
      else
	{
	  volume[cs*8+2*cn]=OUT0BUF[0];
	  volume[cs*8+2*cn+1]=OUT0BUF[1];
	}
    }

   ep0o_dispatch++;
}
/*-------------------------------------------------------------------*/
//! EP0 OUT interrupt
void usb_ep0out_isr(void) interrupt 19
{
	IRQ_PREUSB;
	OUT07IRQ = 0x01;
	handle_set_feature();
	ep0_dispatch=0;
	
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
//! USB-Audio request handler for read
void handle_get_feature(void)  //GET_x
{
  int len;
  uchar n1,n2;
  uchar cs,cn,fuid,ifep;
  
  cs=SETUPDAT[3];
  cn=SETUPDAT[2];
  fuid=SETUPDAT[5];
  ifep=SETUPDAT[4];
  len=SETUPDAT[6];
  OUTA ^= 0x80;
  
  IN0BUF[0]=1; // Never wrong ;-)
  n1=volume[cs*8+2*cn];
  n2=volume[cs*8+2*cn+1];
  

//   if (fuid==2)
     {
       if (SETUPDAT[1]==GET_CUR)
	 {
	   IN0BUF[0]=n1;
	   IN0BUF[1]=n2;
	 }
       if (SETUPDAT[1]==GET_RES)
	 {
	   if (cs==1)
	     {
	       IN0BUF[0]=0;
	       IN0BUF[1]=0;
	     }
	   else
	     {
	       IN0BUF[0]=0x00;
	       IN0BUF[1]=0xb5;
	     }
	 }
        if (SETUPDAT[1]==GET_MAX)
	 {
	   if (cs==1)
	     {
	       IN0BUF[0]=1;
	       IN0BUF[1]=0;
	     }
	   else
	     {
	       IN0BUF[0]=0x00;
	       IN0BUF[1]=0x20;
	     }
	 }
	if (SETUPDAT[1]==GET_MIN)
	 {
	   if (cs==1)
	     {
	       IN0BUF[0]=1;
	       IN0BUF[1]=0;
	     }
	   else
	     {
	       IN0BUF[0]=0x01;
	       IN0BUF[1]=0x80;
	     }
	 }
     }
 
  IN0BC = len;
  // printf("GET FEATURE\n");
}
/*-------------------------------------------------------------------*/
//! Initialize LCD
void init_lcd(void)
{
	uchar n;
	lcd_cont=LCD_CONT;
	
	nvram_bank=NV_BANK0;
	nvram_addr=NV_CONTROL;
	n=read_nvram();
	if ((n&15)==0 || (n==255))
		n=lcd_cont;
	lcd_cont=n|8|0x80;

	_asm;
	mov _dline,#0;
	mov _screen,#0;
	mov dptr,#_xilinx0;
	mov a,_lcd_cont;
	movx @dptr,a;
	inc dptr;
	inc dptr;

	mov a,#0x3c;
	movx @dptr,a;
	lcall _small_wait;

	mov a,#4+2;
	movx @dptr,a;
	lcall _small_wait;

	mov a,#0x2;
	movx @dptr,a;
	lcall _wait_1000

	mov a,#1;
	movx @dptr,a;
	lcall _wait_1000;

	mov a,#0x3c;
	movx @dptr,a;
	lcall _small_wait;

	mov a,#0xc;
	movx @dptr,a;
	lcall _small_wait;

	_endasm;

}
/*-------------------------------------------------------------------*/
//! startup code, init, event loop for keys and USB
void ezusbmain(void)
{
	unsigned int n;
	uchar m,o;
	volatile uchar x;
	uchar ob0;
	fpga_loaded=0;

	ISOCTL = 0;  /* enable ISO endpoints */
	USBBAV = 1;  /* enable autovector, disable breakpoint logic */
	CPUCS=2;     /* CLK24 output enable */
	CKCON = 0x31; /* one external wait state, to avoid chip bugs */
  
	/* TIMER */
	TMOD = 0x21;
	TCON = 0;  
	T2CON= 0;
	TH1 = 0x40;  /* 9600 bauds */
	SCON0=0x40;  // Mode 1
	SCON1=0x40;  // Mode 1
 

	PCON = 0;  /* SMOD0=0 */
  
	/* var init */
	alt_set[0]=0;
	alt_set[1]=0;
	alt_set[2]=0;
	alt_set[3]=0;

	for(x=0;x<32;x++)
		volume[x]=0;

	counter=0;

	monpos=0;
	outpos=0;
	simulate=0;
	ep0_dispatch=0;
	dline=0;
	show_vol=1;
	//fpga_init=0;

	/* Port A: led1 led2 FRD FWR CS OE xil_data xil_cclk*/
	PORTACFG=0x3C;     // enable FRD FWR CS OE
	OEA=0xff;
	OUTA=0xc0;
  
	/* Port B: x I6 I5 I4 x x x x */
	PORTBCFG=0x70;
	OEB=0x8f;
	OUTB=0x0;
  
	/* Port C: ... */
	PORTCCFG = 0xc2;  // TXD on
	OUTC = 0x38;
	OEC = 0x38; 
	
	mute_dac();

#if 0
	_asm;
	mov dptr,#_AUTOPTRH;          ; Init autopointer;
	mov a,_OUT8ADDR>>8;
	movx @dptr,a;
	inc dptr;
	mov a,_OUT8ADDR;
	movx @dptr,a;
	mov dptr,#_AUTODATA;
	
	clr a;
	movx  @dptr,a      ; o8=0;
	movx  @dptr,a      ; o9;
	movx  @dptr,a      ; o10;
	movx  @dptr,a      ; o11;
	movx  @dptr,a      ; o12;
	movx  @dptr,a      ; o13;
	movx  @dptr,a      ; o14;
	movx  @dptr,a      ; o15;
	movx  @dptr,a      ; i8=0;
	mov a,#224/4;
	movx  @dptr,a      ; i9;
	mov a,#(256+416)/4;
	movx  @dptr,a      ; i10;

	movx  @dptr,a      ; i11;
	movx  @dptr,a      ; i12;
	movx  @dptr,a      ; i13;
	movx  @dptr,a      ; i14;
	movx  @dptr,a      ; i15;
	_endasm;
#else

	OUT8ADDR=0;
	OUT9ADDR=0;
	OUT10ADDR=0;
	OUT11ADDR=0;
	OUT12ADDR=0;
	OUT13ADDR=0;
	OUT14ADDR=0;
	OUT15ADDR=0;
 
	IN8ADDR=0;
	IN9ADDR=224/4;
	IN10ADDR=(224+416)/4;
	IN11ADDR=(224+416)/4;
	IN12ADDR=(224+416)/4;
	IN13ADDR=(224+416)/4;
	IN14ADDR=(224+416)/4;
	IN15ADDR=(224+416)/4;
#endif  
	/* setup isochronous EP 8/9 RDI/I2S */
	INISOVAL=1+2;
	OUTISOVAL=1;

	USBPAIR=0x80;  // zero-length ISO + pair out2/3
	init_control();

	for(n=0;n<30;n++) 
	{
		x=xilinx0;
		x=fifo01_clear; // reset fifos
		IN8DATA=x;
	}

#if 1
	USBCS = 0;  /* disconnect from USB */
	for (n = 0; n < 12000; n++)
	{
		wait_0_1ms();
	}
#endif 

	OUTA&=~0x40;

	USBCS = 2;  /* 8051 handles control */
	USBCS = 6;  /* reconnect, 8051 handles control */

	IN07VAL=4+1;   // enable EP2&0
	OUT07VAL=4+1;
	IN07IEN=0;     // enable EP2 int
	OUT07IEN=4+1; 
 
	IE = 0x80;     //global int enable 
  
	EIP= 0x10;      // Int6/4 have high priority 
	USBIEN = 1|2|8;   // enable SUDAV interrupt 
	FASTXFR=0x88;     // FRD Mode 01
	USBIRQ = 0x08;

	EIE = 0x01+1*16 ; //+ 0x1c; /* enable USB interrupts +int456*/

	/* enable interrupts */

	simulate=1;

	ep2_out_req=0;
	ep2_in_req=0; 
	volume[10]=0x0;
        volume[12]=0x0;
        volume[19]=0xf0;
        volume[21]=0xf0;
//	set_volume();
	last_key=0;
	subch=0;
	prog_set=0;
	key=0;
	last_locked_key=0;
	for(m=0;m<16;m++)
	{
		names[m][0]=0;
		subchannels[m]=m+1;
	}
	ensemble[10]=0;

	for(m=0;m<16;m++)
	{
		names[0][m]=version_str[m];
		ensemble[m]=vendor_str[m];
	}

	num_sub=15;
	blink_toggle=0;

//	while(1);
//		IN2BC=64;
		
	for (;;)   // Main service loop
	{
		USBIEN = 1|2;   // enable SUDAV interrupt 

		if (ep2_out_req)
		{
			//  printh(OUT2BC);
			if (OUT2BC)
			{
				if (blink_toggle)
					OUTA|=0x80;
				ob0=OUT2BUF[0];
				if (ob0==0x2c)   // Init FPGA/DAC
				{	
					x=xreset;
					simulate=0;
					xilinx0=0x40;
					dac_startup();
					
					wait_1000();
					init_lcd();

					show_name();

					define_all_chars();
					wait_1000();
					fpga_loaded=1;
					wait_1000();
					OUTA&=~0xc0;
					OUTC&=~0x30;

				}
				else if (ob0==0x2a)  // Clear FPGA
				{			
					fpga_loaded=0;		
					mute_dac();
					xilinx0=0x50;
					OUTA&=~0xc0;
//					printf("Clear");

					OUTC = 0x30;
					wait_0_1ms();
					OUTC = 0x38;	   
					for(o=0;o<10;o++)
					{
						wait_0_1ms();
					}
				}
				else if  ((ob0==0x2b)&&(OUT2BC==64)) // DOWNLOAD FPGA
				{
					xilinx0=0x40;
					download_fpga();
					xilinx0=0x40;
					key=0;
				}

				else  if (ob0==0x10) // read dfire
				{
					IN2BC=0;
					read_module_buf();					
					//      OUTA^=0x80;
				}
				else if (ob0==0x11) // write dfire
					write_module_buf();
				else if (ob0==0x12) 
					write_i2c();
				else if (ob0==0x13)
				{  
					xilinx0=OUT2BUF[1]; // Set Controlreg
				}
				else if (ob0==0x14)
				{
					printf(OUT2BUF+4);
				}
				else if (ob0==0x15)
				{
					select_prog(OUT2BUF[1],OUT2BUF[2]);
				}
				else if (ob0==0x16)
				{
					set_name();
				}							
				else if (ob0==0x17) 
					read_i2c();
				else if (ob0==0x18) 
					read_ee();
				else if (ob0==0x19)  // get special key
				{
					IN2BUF[0]=last_locked_key;
					IN2BC=1;
					last_locked_key=0xff;
				}
		      
			}

			EIE=0x0;
			ep2_out_req=0;
			OUT2BC=0;       // arm for next transfer
			EIE = 0x01+1*16 ; //+ 0x1c; /* enable USB interrupts +int456*/
			continue;
		}

		if (ep0o_dispatch && (n&15)==0)
		{
//			if (ensemble[0]!=0)
			if (ep0o_dispatch==1)
				set_volume();
			ep0o_dispatch--;
		}

#if 1

		if ((n&255)==128)
		{
			last_key=0;

			if (key)
			{
				n=0;
				OUTA|=0x80;
			}

			poscursor(0,0);
			ob0=volume[19]+0x80;

			if ((key==KEY_VOLPLUS)&&(ob0<0xa0))
			{
				show_vol=1;
				volume[19]+=2;
				volume[21]=volume[19];
				volume[10]=volume[12]=0;
				set_volume();	    
				key=0;
			}
			else if ((key==KEY_VOLMINUS)&&(ob0>4))
			{
				show_vol=1;
				volume[19]-=2;
				volume[21]=volume[19];
				volume[10]=volume[12]=0;
				set_volume();
				key=0;				
			}
			else if ((key==KEY_PROGPLUS)&&(!last_key))
			{
				subch=subch+1;
				if (subch>num_sub)
					subch=0;
				select_prog(subch,0);
				key=0;
			}
			else if ((key==KEY_PROGMINUS)&&(!last_key))
			{
				subch=subch-1;
				if (subch==255)
					subch=num_sub;
				select_prog(subch,0);
				key=0;
			}
			else if (key!=0)
			{
//				if (!last_locked_key)
				{
					poscursor00();
					printf("   L O C K E D   ");
					last_locked_key=key;
				}
				if (key==KEY_MENU)
					blink_toggle=1-blink_toggle;
				key=0;
			}
		}
#endif
		wait_0_1ms();
		OUTA&=0x7f;
		OUTB=0x8;


		if (blink_toggle)
			if (((n&255)==0) && ((n>>8)&3)==1)
				OUTA^=0x40;

		if ((n>>8)>=16)
		{    
			n=0;
			OUTA&=~0x80;
			show_vol=0;
			show_name();
#if 0
			poscursor00();

			printh(volume[11]); printh(volume[10]);
			printf(" ");
			printh(volume[13]); printh(volume[12]);
	  
			poscursor(0,1);
			printh(volume[19]); printh(volume[18]);
			printf(" ");
			printh(volume[21]); printh(volume[20]);
#endif  
		}
		else
			n++;

	}
}
