// dabusb_a.c, STANDALONE
// 140400 GA
//
//
//   $Id: dabusb_a.c,v 1.24.2.32 2005/01/15 01:29:39 acher Exp $

#include "regezusb.h"
#pragma SAVE
#pragma NOOVERLAY

/* Memory map (preliminary):

   0000-1fff  EZ-USB code/data ram
   2000-27ff  EZ-USB isochronous fifos/extra RAM (STANDALONE only)
   2800-7b3f  reserved for future (FPGA mirrored)
   7b40-7fff  EZ-USB bulk buffer, HW registers
   8000-83ff  FPGA reserved     [a(11:10)=00]
   8400-87ff  D-FIRE DP-RAM CS  [a(11:10)=01]
   8800-8fff  FPGA reserved     [a(11:10)=1x]
   9000-ffff  reserved (FPGA mirrored)
*/ 

#define MIN_SUBCH 5
#define MIN_ENSEMBLE 31
#define MIN_BAND 0

typedef unsigned char uchar;

#include "io.h"
#include "control_a.h"

#include "variables_a.h"
#ifdef KEY12
#define VENDOR_STR "TerraTec"
#define VERSION_STR "DR Box 1 V1.18ST" 
#else
#define VENDOR_STR "BayCom"
#define VERSION_STR "DABUSB V1.184ST"
#endif

extern void download_fpga(void);

extern void decrement_subch(void);
extern void increment_subch(void);

extern void default_key_handler(void);
extern void handle_scan_menu(void);
extern void handle_equalizer_menu(void);
extern void enter_browse_menu(void);
extern void enter_scan_menu(void);
extern void handle_spdif_menu(void);
extern void handle_lcd_menu(void);
extern void handle_lbmode_menu(void);
extern void get_channel_info(void);
extern void store_lcd_cont(void);
extern void show_menutext(void);


// Main menu
#define MENU_NORMAL 0
#define MENU_QUALITY 1
#define MENU_BROWSE 2
#define MENU_SCAN 3
#define MENU_EQUALIZER 4
#define MENU_SETTINGS 5

// settings menu (major 5)
#define MENU_SPDIF 0
#define MENU_LCD 1

// equalizer menu (major 4)
#define MENU_BASS 0
#define MENU_MID 1
#define MENU_TREBLE 2


/*-------------------------------------------------------------------*/
// STATIC VARIABLES in internal RAM


extern uchar key,dline;

uchar subch, band, channel, storage, bank, sidh, sidl;
uchar lbmode;

uchar current_subch, current_band, current_channel;


uchar last_key,xkey,timer,keytest,timerset;
uchar lcd_cont, menu, quality;
uchar num_sub,tn,timeout, num_all;
uchar select_rdi_cnt;
uchar xvolume, eq_val[3], supress_bar;
uchar quality_mode;

uchar hour,min,sec,utc_off;

unsigned int timer_n;
//extern void (*menu_check_handler)(void);
void					get_audio_info(void);
code uchar menu_text_0[6]={128,
			   129,
			   STR_BROWSEBANKS,
			   STR_SCAN,
			   STR_EQUALIZER,
			   STR_SETTINGS};

code uchar menu_text_4[3]={STR_BASS,
			   STR_MID,
			   STR_TREBLE};

code uchar menu_text_5[4]={STR_DIGITALOUTPUT,
			   STR_LCD, STR_LBMODE0, STR_VERSION};

#ifndef KEY12
code uchar next_table[6*6]={
	0x01,0x03,0x00,0x00,0x00,0x00,  // Main menu
	0x10,0x10,0x10,0x10,0x10,0x10,  // quality
	0x20,0x20,0x20,0x20,0x20,0x20,  // browse
	0x30,0x30,0x30,0x30,0x30,0x30,  // scan
	0x41,0x42,0x04,0x00,0x00,0x00,   // EQ
	0x51,0x52,0x53,0x05,0x00,0x00   // settings
};
#else
code uchar next_table[6*6]={
	0x01,0x02,0x03,0x04,0x05,0x00,  // Main menu
	0x10,0x10,0x10,0x10,0x10,0x10,  // quality
	0x20,0x20,0x20,0x20,0x20,0x20,  // browse
	0x30,0x30,0x30,0x30,0x30,0x30,  // scan
	0x41,0x42,0x04,0x00,0x00,0x00,   // EQ
	0x51,0x52,0x53,0x05,0x00,0x00   // settings
};
#endif
char hack_def(void)
{

	_asm;
	.globl _sids;     // make global
	.globl _names;
	.globl _str_table;

	.area ZEUG (ABS);
	.org 0x1c40;
_str_table:
	.dw _ensemble;   // 0
	.dw _current_name; //1

	.dw str_tuning;  // 2
	.dw str_null;    // 3 
	.dw str_scan;    // 4
	.dw str_ginfo;   // 5
	.dw str_stored;  // 6
	.dw str_numsubs; // 7

	.dw str_swap_with;   // 8	NU
	.dw str_bandsel; //9 
	.dw str_digitaloutput; //a
	.dw str_audiordi; // b
	.dw str_browsebanks; //c
	.dw str_settings; //d
	.dw str_bass;
	.dw str_mid;

	.dw str_treble;   //10
	.dw str_band3;
	.dw str_lband;
	.dw str_nostorage;
	.dw str_browsekeys1; //14
	.dw str_browsekeys2; //15
	.dw str_lcd;  //16
	.dw str_scanenter;  //17
	.dw str_band3lband; // 18
	.dw str_equalizer;  // 19
	.dw str_select;  // 1a
	.dw str_version;  // 1b
	.dw str_vendor;  // 1c
	.dw str_abort;
	.dw str_swap_with_1;
	.dw str_clear_all;
	.dw str_lbmode0;  //20
	.dw str_lbmode1;  //21
str_select:
	;       01234567890122345
	.ascii "        Select \007\000";
str_tuning:
	.ascii "Tuning  \000";
str_null:
	.ascii "\000";
str_scan:
	.ascii "Scan\000";
str_ginfo:
	.ascii "Get Info    \000";
str_swap_with:
	.ascii "Choose\004\005, \006Swap!\000";
str_stored:
	.ascii "Stored!      \000";
str_numsubs:
	.ascii "Channels \000";

str_bandsel:
	.ascii "\006\007 Band\004 Tune\005\000";
str_digitaloutput:
	.ascii " Digital Output\000";
str_audiordi:
	.ascii "\006 Audio    RDI \007\000";
str_browsebanks:
	.ascii "Browse Banks\000";
str_settings:
	.ascii "Settings\000";
str_bass:
	.ascii "      Bass\000";
str_mid:
	.ascii "       Mid\000";
str_treble:
	.ascii "     Treble\000";
str_band3:
	.ascii "Band 3\000";
str_lband:
	.ascii "L-Band\000";
str_nostorage:
	.ascii "No Storage!\000";
str_browsekeys1:
	.ascii "\004 P+ M=Exit P- \005\000";
str_browsekeys2:
	.ascii "\006 Swap    Tune \007\000";
str_lcd:
	.ascii "LCD Contrast\000";
str_scanenter:
	.ascii "Scan=\006\007  Enter=\004\000";
str_band3lband:
	.ascii "\006 Band3 L-Band \007\000";
str_equalizer:
	.ascii "Equalizer\000";
str_version:
	.ascii VERSION_STR;
	.db 0;
str_vendor:
	.ascii VENDOR_STR;
	.db 0;
str_abort:
	.ascii "Aborted...\000";
str_swap_with_1:
	.ascii "Swap with...\000";
str_clear_all:
	.ascii "Clear all? OK=\006\000"; 
str_lbmode0:
	.ascii "L-Band Mode\000";
str_lbmode1:
	.ascii "\006 Canada  CEPT \007\000";

	.area CSEG    (CODE)
	_endasm;
		return bitrate_idx[0][0]; // needed for hack!
}

/*-------------------------------------------------------------------*/
// IRQ macros
#define IRQ_PRE     _asm \
                    push _DPS \
                    _endasm; \
                    DPS=0;             

#define IRQ_PREUSB  _asm \
                    push _DPS \
                    _endasm; \
                    DPS=0; \
                    _asm \
	            mov a,_EXIF \
                    clr acc.4 \
                    mov _EXIF,a \
                   _endasm;


#define IRQ_POST    _asm \
	            pop _DPS \
                    _endasm;
/*-------------------------------------------------------------------*/
void hack(void)
{
	// make some variables global
	_asm;
	.globl _nvram_bank;
	.globl _nvram_val;
	.globl _nvram_addr;
	_endasm;
}
/*-------------------------------------------------------------------*/
void timer0_isr(void) interrupt 2
{
	
	IRQ_PRE;
	_asm;
	clr IE.7
	push acc;
	push ar0;

	mov a,_timer;
	inc a;
	anl a,#1;
	mov _timer,a
;	jnz timer_not_zero;
	inc _keytest;	
       	lcall _get_key;
timer_not_zero:
	inc _timerset;
	mov dpl,_timer_n;
	mov dph,_timer_n+1;
	inc dptr;
	mov _timer_n,dpl;
	mov _timer_n+1,dph;
	push ar1;
	lcall _show_bank_leds;
	pop ar1;
       	setb IE.7;

	pop ar0;
	pop acc;
	_endasm;
	IRQ_POST;
}
/*-------------------------------------------------------------------*/
void clrtn(void)
{
	_asm;
	mov a,_IE;
	anl a,#~2;
	mov _IE,a;
	clr a;
	mov (_timer_n + 1),a;
	mov  (_timer_n),a;
	mov a,_IE;
	orl a,#2;
	mov _IE,a;
	_endasm;
}
/*-------------------------------------------------------------------*/
extern void wait_0_1ms(void);
/*-------------------------------------------------------------------*/
void mute_dac(void)
{
	Ci2c_out3(0x02,0,0);
	Ci2c_out2(0x03,0x40);
}
/*-------------------------------------------------------------------*/
void init_dac()
{ 
	xilinx0=64+(lcd_cont&(~DAC_RESET));  // DAC Reset
	lcd_cont|=0x10;
	xilinx0=64+lcd_cont;
	wait_0_1ms();
	Ci2c_out2(0x01,0x18   );
	Ci2c_out3(0x02,0x00,0x00);
	wait_0_1ms();
	Ci2c_out2(0x03,0x44);
}
/*-------------------------------------------------------------------*/
void set_volume(void)
{      	
	Ci2c_out3(0x02,xvolume,xvolume);
      
	if (!supress_bar)
	{
		clr_line1();
		Cputc('V');
		printi(xvolume,2);
		Cbargraph(1+60/4,4,3);
		Cbargraph(1+xvolume/4,4,1);
	}
}

/*-------------------------------------------------------------------*/
void equalizer(uchar show);

/*-------------------------------------------------------------------*/
void init_lcd(void)
{
	screen=0;
	_asm;
	mov dptr,#_xilinx0;
	mov a,_lcd_cont;
	movx @dptr,a;
	inc dptr;
	inc dptr;

	mov a,#0x3c;
	movx @dptr,a;
	lcall _small_wait;

	mov a,#4+2;
	movx @dptr,a;
	lcall _small_wait;

	mov a,#1;
	movx @dptr,a;
	lcall _wait_3000;

	mov a,#2;
	movx @dptr,a;
	lcall _wait_3000;

	mov dptr,#_xilinx2;
	mov a,#0x3c;
	movx @dptr,a;
	lcall _small_wait;

	mov a,#0xc;
	movx @dptr,a;
	lcall _small_wait;
	

	mov a,#STR_VENDOR;
	lcall _printf_index00_acc;
	mov a,#STR_VERSION;
	lcall _printf_index01_acc;
	mov a,#STR_VENDOR;
	lcall _printf_index00_acc;
	_endasm;

}
/*-------------------------------------------------------------------*/
void write_nvram(void);
uchar read_nvram(void);
extern init_debug(void);
/*-------------------------------------------------------------------*/
extern void i2c_stop_read(void);
extern void i2c_set_addr_read(void); // ASM-Binding! Addr-H in r2, L in r3
/*-------------------------------------------------------------------*/
void init_stuff(void)
{
	_asm;
	mov a,#0;
	mov _TL0,a;
	mov _PCON,a;
	mov _dline,a;

	mov dptr,#_PORTACFG;       // 7F93
	mov a,#0x3c;
	movx @dptr,a;
	inc dptr;                 // PORTBCFG
	mov a,#0x70;
	movx @dptr,a;
	inc dptr;                 // PORTCCFG
	mov a,#0xc2;
	movx @dptr,a;

	inc dptr;                 // OUTA 7F96
	mov a,#0x80;
	movx @dptr,a;
	inc dptr;                 // OUTB
	mov a,#0x00;
	movx @dptr,a;
	inc dptr;                 // OUTC
	mov a,#0x38;
	movx @dptr,a;

	mov dptr,#_OEA;
	mov a,#0xff;
	movx @dptr,a;
	inc dptr; // OEB
	mov a,#0x8f;
	movx @dptr,a;
	inc dptr; // OEC
	mov a,#0x38;
	movx @dptr,a;

	clr a;
	mov dptr,#_IN07VAL;     // 7FDE
	movx  @dptr,a;
	inc dptr;               // OUT07VAL 7FDF
	movx  @dptr,a;
	inc dptr;               // INISOVAL 7FE0
	movx  @dptr,a;
	inc dptr;               // OUTISOVAL 7FE1
	movx  @dptr,a;

	mov dptr,#_IN07IEN;
	movx  @dptr,a;
	inc dptr;               // OUT07IEN
	movx  @dptr,a;

	mov a,#255;
	mov _current_band,a;
	mov _current_channel,a;
	mov _current_subch,a;

	clr a;
	mov _bank,a;
	mov _last_key,a;
	mov _menu,a;
	mov _keytest,a;
	mov _xkey,a;
	_endasm;
}
/*-------------------------------------------------------------------*/
void scroll_menutext(void)
{
	screen=16;
	show_menutext();
	shift_left();
	screen=0;
	shift_right();
	show_menutext();	
}
/*-------------------------------------------------------------------*/
void wait_and_show(void)
{
	scroll_menutext();
	clrtn();
	wait_key_release0();
}
/*-------------------------------------------------------------------*/
void restore_menu0(void)
{
	menu=MENU_NORMAL;
	show_menutext();
	clrtn();
	wait_key_release0();
}
/*-------------------------------------------------------------------*/
void ezusbmain(void)
{
//	uchar m,o,i;
//	volatile uchar x;
	uchar m;
	uchar keytime;
	
	USBCS=4;

	ISOCTL = 1;   /* disable ISO endpoints, enable extra 2K */
	USBBAV = 0;   /* enable autovector, disable breakpoint logic */
	CPUCS=2;      /* CLK24 output enable */
	CKCON = 0x39; /* one external wait state, to avoid chip bugs */
	
	init_stuff();

	USBPAIR=0x00;

	xvolume=40;

	eq_val[0]=eq_val[1]=eq_val[2]=128;
	screen=0;
	init_control();
	lcd_cont=LCD_CONT;
	init_lcd();

//	init_debug();
	setz();
	
	IE = 0x80;     //global int enable 
	EIP= 0x10;      // Int6/4 have high priority 
	USBIEN = 0;   // enable SUDAV interrupt 
	FASTXFR=0x88;     // FRD Mode 01
	USBIRQ = 0x08;
	
	EIE = 0x01 ; //+ 0x1c; /* enable USB interrupts +int456*/
	
	cpubuf[0]=0;
	cpubuf[0]=42;
	cpubuf[1]=0;

	if (cpubuf[0]!=42)
		download_fpga();
	init_dac();
	init_lcd();
	define_all_chars();

#ifndef MINIMAL
#warning "default configuration"
	
	CNVC_2_scratch(0);
	Cscratch_2_current(0);

	if (storage!=255)
		recall_program(storage,0);
	else
		bank=0;
//	putc('2');

	nvram_bank=NV_BANK0;
	nvram_addr=NV_CONTROL;
	m=read_nvram();
	if ((m&15)==0 || (m==255))
		m=lcd_cont;
	lcd_cont=m;
	xilinx0=lcd_cont;
	nvram_addr=NV_LBMODE;
	lbmode=read_nvram();

#else
#warning "minimal configuration"    
	subch=MIN_SUBCH;
	channel=MIN_ENSEMBLE;
	band=MIN_BAND;
	storage=255;
	bank=0;
#endif

	supress_bar=1;
	equalizer(0);
//	putc('3');
	set_volume();
//	putc('4');
	supress_bar=0;

	setz();
	wait_3000();

	OUTC=0x8;

	recall_chan_sub();

	TMOD=0x01;  // Set TIMER0
	TCON=0x10;
	timeout=160;

	keytime=0;

	clrtn();	
	tn=0;
	key=0;
	select_rdi_cnt=0;
	quality_mode=0;

	for (;;)   // Main service loop
	{	
#ifdef MINIMAL
		if ((num_sub==16 || num_sub==0) && (((tn)&63)==63))
			OUTA^=0x80;
#endif

		if (timerset)
		{
			timerset=0;
			tn++;
		}
		timeout=160;
	
		menu_timer_calls1();  // reset menu&bank LEDs
		{
//			menu_bank_menu();

			if (key==KEY_MENU)
			{
				menu=next_table[(menu>>4)*6+(menu&15)];
				wait_and_show();
			
			}		

			if (((menu&15)>=0x2) && 
			    ((menu>>4)==0) &&
			    (key==KEY_VOLPLUS))
			{
				if ((menu&15)==MENU_BROWSE)
				{
					enter_browse_menu();
					menu=MENU_NORMAL;
				}
				else if ((menu&15)==MENU_SCAN)
				{
					enter_scan_menu();
					menu=MENU_NORMAL;
				}
				else
					menu=menu<<4;
				wait_and_show();
			}

			if (menu<2)
			{
				menu_bank_menu();
				default_key_handler();
			}

			if ((menu>>4)==MENU_EQUALIZER)
				handle_equalizer_menu();

			if (menu==0x50)
				handle_spdif_menu();

			if (menu==0x51)
				handle_lcd_menu();
			if (menu==0x52)
				handle_lbmode_menu();
			key=0;			

			poscursor00();
//			printh(menu);
//
			switch (menu)
			{
			case MENU_NORMAL: // Standard
#if 1 //ndef MINIMAL
				if (!key && (tn&7)==7 && tn>10)
					Cshow_name(subch);
#endif
				break;

			case MENU_QUALITY: // Quality
#ifndef MINIMAL
				if ((tn&3)==3)	
				{
//					get_audio_info();
					show_dabstate();
				}
//				break;
#endif			
			default:
				break;
			}	
		}

#ifdef MINIMAL
		subch=MIN_SUBCH;
#endif
#if 1 // ENABLE FIG
//		OUTB=0x8;
		if (menu==MENU_NORMAL || menu==MENU_QUALITY)
		{
			if (quality_mode==0)
			{
				get_channel_info();
				
				if ((tn&15)==7)
					rdi_select_all();
			}


#ifndef MINIMAL
			acquire_figs();  // ASM
#endif	
			wait_1000();
		}	
		
#endif	
		
	} // for-loop
}
