/* DABUSB descriptor 
   Audio control, audio streaming, HW control, RDI streaming
   new version

  $Id: descriptor.c,v 1.2.2.4 2001/06/20 15:29:47 acher Exp $

*/
#define USB_DT_DEVICE                   0x01
#define USB_DT_CONFIG                   0x02
#define USB_DT_STRING                   0x03
#define USB_DT_INTERFACE                0x04
#define USB_DT_ENDPOINT                 0x05

#define USB_IFC_AUDIO                   0x01

#define USB_IFSC_UNDEFINED             0x00
#define USB_IFSC_AUDIOCONTROL          0x01
#define USB_IFSC_AUDIOSTREAMING        0x02
#define USB_IFSC_MIDISTREAMING         0x03

#define USB_CS_UNDEFINED               0x20
#define USB_CS_DEVICE                  0x21
#define USB_CS_CONFIGURATION           0x22
#define USB_CS_STRING                  0x23
#define USB_CS_INTERFACE               0x24
#define USB_CS_ENDPOINT                0x25

#define USB_AC_DESCRIPTOR_UNDEFINED    0x00
#define USB_AC_HEADER                  0x01 
#define USB_AC_INPUT_TERMINAL          0x02
#define USB_AC_OUTPUT_TERMINAL         0x03
#define USB_AC_MIXER_UNIT              0x04
#define USB_AC_SELECTOR_UNIT           0x05
#define USB_AC_FEATURE_UNIT            0x06
#define USB_AC_PROCESSING_UNIT         0x07
#define USB_AC_EXTENSION_UNIT          0x08

#define USB_AS_DESCRIPTOR_UNDEFINED    0x00
#define USB_AS_GENERAL                 0x01
#define USB_AS_FORMAT_TYPE             0x02
#define USB_AS_FORMAT_SPECIFIC         0x03

#define USB_FU_MUTE_CONTROL            0x01
#define USB_FU_VOLUME_CONTROL          0x02

#define USB_FT_FORMAT_TYPE_I           0x01
#define USB_EP_GENERAL                 0x01

/*-----------------------------------------------------------------------*/

static  unsigned char usb_device_descriptor[] = 
{ 
  0x12,        // bLength 
  USB_DT_DEVICE ,        // bDescriptorType 
  0x00, 0x01,  // bcdUSB 
  0x00,        // bDeviceClass (independent interfaces) 
  0x00,        // bDeviceSubClass (unused)
  0x00,        // bDeviceProtocol (inteface based) 
  0x40,        // bMaxPacketSize0 
#if 0
  0x47, 0x5,  // idVendor 
  0x99,0x99, // idProduct 
#else
  0xcd,0xc, 
  0x1,0x80,
#endif
  0x00, 0x01,  // bcdDevice 
  0x00,        // iManufacturer 
  0x00,        // iProduct 
  0x00,        // iSerialNumber 
  0x01,         // bNumConfigurations (1) 

  0,0,0,0,0,0,0
};
/*-----------------------------------------------------------------------*/

//#define LINUX

#define CF0_DESC(numi,para) \
 0x09,                         /* bLength */ \
  USB_DT_CONFIG,                /* bDescriptorType */  \
  (9+sizeof((char[]){para}))&255,((9+sizeof((char[]){para}))>>8)&255, \
  numi,                         /* bNumInterfaces */ \
  0x01,                         /* bConfigurationValue */\
  0x00,                         /* iConfiguration (unused) */\
  0x40,                         /* bmAttributs, not selfpowered */\
  0x00,                          /* MaxPower */\
  para

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// standard AC IF descriptor
#define IF0_STD_DESC \
  0x09, \
  USB_DT_INTERFACE, \
  0,                             /* index IF */ \
  0,                             /* a index set */ \
  0,                             /* 0 EP        */ \
  USB_IFC_AUDIO, \
  USB_IFSC_AUDIOCONTROL, \
  0, \
  0 

#ifndef LINUX
#define IF0X_CS_DESC(para) \
   0x09,                  \
  USB_CS_INTERFACE,       \
  USB_AC_HEADER,          \
  0x00,0x01,              \
  (9+sizeof((char[]){para}))&255,((9+sizeof((char[]){para}))>>8)&255, \
  1,                          /* num stream IF */ \
  1,                          /* with IF 1 */ \
  para 

#define IT1X \
  0x0c, 0x24,2,1, 0x10,7  ,0,2,3,0,0,0, /* IT1 */ \
  0x0d, 0x24,6,/*ID */0x03,/* SRC */ 0x01,0x2, 0x3,0, 0x3,0, 0x3,0 ,0, /*FU */ \
  9,0x24,3, /* ID */ 2,1,1,0,1,0,   /* OT USB */ \
  9,0x24,3, /* ID */ 4,2,3,0,3,0,   /* OT Headphone */ \
  9,0x24,3, /* ID */ 5,3,6,0,1,0,   /* OT Lineout */ \

#else
#define IF0X_CS_DESC(para)
#define IT1X
#endif

#ifdef LINUX
#define USE_AS1 0
#else
#define USE_AS1 1
#endif

#if USE_AS1==1
#define IF1X_AS0_STD_DESC \
  0x09, \
  USB_DT_INTERFACE, \
  1,                       /* if# */\
  0,                       /* as#0 */\
  0,                       /* 0 EP */\
  USB_IFC_AUDIO, \
  USB_IFSC_AUDIOSTREAMING, \
  0, \
  0,
#else
#define IF1X_AS0_STD_DESC
#endif

#define IF1X_AS1_STD_DESC \
  0x09 , \
  USB_DT_INTERFACE, \
  1,                     \
  USE_AS1,                  \
  1,                      \
  USB_IFC_AUDIO,\
  USB_IFSC_AUDIOSTREAMING,\
  0,\
  0, 

#ifndef LINUX 
#define IF1X_AS1_CS_DESC \
  7,0x24,1,/*SRC*/2,1,1,0,  /* CS General */\
  0xb,0x24,2,1,2,2,0x10,1,0x80,0xbb,0,  /*Format */\
  9,5,0x88,5,0x00,0x1,1,0,0, /* Standard EP */\
  7,0x25,1,0,0,0,0, /* CS EP */ 

#else
#define IF1X_AS1_CS_DESC \
   9,5,0x88,5,0xe0,0,1,0,0, /* Standard EP */
#endif

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define IF2_DESC\
  0x09, \
  USB_DT_INTERFACE, \
  2,                    /* index IF */\
  0,                    /* a index set*/ \
  2,                    /* 2 EPs */\
  0x0, \
  0x0, \
  0x0 , \
  0, \
  0x07, \
  USB_DT_ENDPOINT, \
  0x82,                  /* IN-2 */\
  2,                     /* bulk */\
  0x40,0x00,             /* 64 bytes */\
  0,                     /* no interval */\
  0x07, \
  USB_DT_ENDPOINT, \
  0x02,                  /* OUT-2 */\
  2,                     /* bulk */\
  0x40,0x00,             /* 64 bytes */\
  0  ,                   /* no interval  */

#if 1
#ifndef LINUX
#define IF2A_DESC\
  0x09, \
  USB_DT_INTERFACE, \
  2,                    /* index IF */\
  1,                    /* a index set*/ \
  3,                    /* 0 EPs */\
  0x0, \
  0x0, \
  0x0 , \
  0, \
  0x07, \
  USB_DT_ENDPOINT, \
  0x82,                  /* IN-2 */\
  2,                     /* bulk */\
  0x40,0x00,             /* 64 bytes */\
  0,                     /* no interval */\
  0x07, \
  USB_DT_ENDPOINT, \
  0x02,                  /* OUT-2 */\
  2,                     /* bulk */\
  0x40,0x00,             /* 64 bytes */\
  0  , \
  9,5,0x89,5,0xa0,0x1,1,0,0,  /* iso (384+32) */\

#else
#define IF2A_DESC
#endif
#endif
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


#define IF0X_DESC \
 IF0_STD_DESC,IF0X_CS_DESC( IT1X )

#define IF1X_DESC \
  IF1X_AS0_STD_DESC IF1X_AS1_STD_DESC IF1X_AS1_CS_DESC


#define IF_DESC \
 IF0X_DESC  IF1X_DESC IF2_DESC IF2A_DESC
 
#define CF_DESC CF0_DESC(3,IF_DESC)

//########################################################################
//########################################################################
//########################################################################
//########################################################################
//########################################################################
//########################################################################
//########################################################################
//########################################################################
//########################################################################
//########################################################################





static  unsigned char usb_config0_descriptor[] = 
{
   CF_DESC
};

void po(unsigned char* s, int l)
{ 
  int n,m;
   for(n=0;n<l;n+=8)
    printf("0x%x,0x%x,0x%x,0x%x,0x%x,0x%x,0x%x,0x%x,\n",
	   s[n],
	   s[n+1],
	   s[n+2],
	   s[n+3],
	   s[n+4],
	   s[n+5],
	   s[n+6],
	   s[n+7]);
   printf("};\n");
}
int main(int arg, char** argv)
{
 
  printf("static const unsigned char usb_config0_descriptor[] = {\n");
  po(usb_config0_descriptor,sizeof(usb_config0_descriptor));

  printf("static const unsigned char usb_device_descriptor[] = {\n");
  po(usb_device_descriptor,sizeof(usb_device_descriptor));
  
  exit(0);
}






