/* DABUSB descriptor 
   Audio control, audio streaming, HW control, RDI streaming
   010799 GA

  $Id: descriptor.h,v 1.1 1999/08/23 18:07:10 acher Exp $

*/

/* 
Configuration 0:
---------------------------------------
Interface 0 (Audio control, AC-EP#0(I/O))            (I2S-Control)

             +--(FU2)-----(OT3)  
             |
  (IT1)------+------------(OT4)
             |
             +------------(OT5)

IT1: 0x02-0x0710 (radioreceiver)
FU2: 0x06-0x01(mute)/0x02(volume)
OT3: 0x03-0x0302(headphone)
OT4: 0x03-0x0603(line out)
OT5: 0x03-0x0101(USB streaming)

---------------------------------------
Interface 1: (AUDIO_STREAMING, #EP:1)               (I2S-Stream)

AS0:{ 0 EP (AUDIO, AUDIO_STREAMING) }


AS1:{ 1 EP (AUDIO, AUDIO_STREAMING)
      OT-link:5
      TYPE-1 format, 48kHz,2*16 }

{ For future versions
AS2:{ 1 EP (AUDIO, AUDIO_STREAMING)
      OT-link:5
      TYPE-1 format, 24kHz,2*16 }}

EP:{ EP IN-8, ISO/ASYNC, >192Byte, interval 1;
     No SR-control, no lock delay }

---------------------------------------
Interface 2: (Vendor specific, #EP:2)                (uC/FPGA/DFIRE/-CONTROL)
EP:{ 1 EP OUT-1, BULK, 64Byte, interval 0;
     1 EP IN-1, BULK. 64Byte, interval 0}
---------------------------------------
Interface 3:

AS0: { 0 EP ???}

AS1: { 1 EP IN-9, ISO, >384Byte, interval 1}

 */

#define USB_DT_DEVICE                   0x01
#define USB_DT_CONFIG                   0x02
#define USB_DT_STRING                   0x03
#define USB_DT_INTERFACE                0x04
#define USB_DT_ENDPOINT                 0x05

#define USB_IFC_AUDIO                   0x01

#define USB_IFSC_UNDEFINED             0x00
#define USB_IFSC_AUDIOCONTROL          0x01
#define USB_IFSC_AUDIOSTREAMING        0x02
#define USB_IFSC_MIDISTREAMING         0x03

#define USB_CS_UNDEFINED               0x20
#define USB_CS_DEVICE                  0x21
#define USB_CS_CONFIGURATION           0x22
#define USB_CS_STRING                  0x23
#define USB_CS_INTERFACE               0x24
#define USB_CS_ENDPOINT                0x25

#define USB_AC_DESCRIPTOR_UNDEFINED    0x00
#define USB_AC_HEADER                  0x01 
#define USB_AC_INPUT_TERMINAL          0x02
#define USB_AC_OUTPUT_TERMINAL         0x03
#define USB_AC_MIXER_UNIT              0x04
#define USB_AC_SELECTOR_UNIT           0x05
#define USB_AC_FEATURE_UNIT            0x06
#define USB_AC_PROCESSING_UNIT         0x07
#define USB_AC_EXTENSION_UNIT          0x08

#define USB_AS_DESCRIPTOR_UNDEFINED    0x00
#define USB_AS_GENERAL                 0x01
#define USB_AS_FORMAT_TYPE             0x02
#define USB_AS_FORMAT_SPECIFIC         0x03

#define USB_FU_MUTE_CONTROL            0x01
#define USB_FU_VOLUME_CONTROL          0x02

#define USB_FT_FORMAT_TYPE_I           0x01
#define USB_EP_GENERAL                 0x01


/*-----------------------------------------------------------------------*/
static const short usb_device_descriptor[] = 
{ 
  0x12,        // bLength 
  USB_DT_DEVICE ,        // bDescriptorType 
  0x00, 0x01,  // bcdUSB 
  0x00,        // bDeviceClass (independent interfaces) 
  0x00,        // bDeviceSubClass (unused)
  0x00,        // bDeviceProtocol (inteface based) 
  0x40,        // bMaxPacketSize0 
  0x47, 0x5,  // idVendor 
  //  0x31,0x21,  // idProduct 
  0x99,0x99,
  0x01, 0x01,  // bcdDevice 
  0x00,        // iManufacturer 
  0x00,        // iProduct 
  0x00,        // iSerialNumber 
  0x01         // bNumConfigurations (1) 
};
/*-----------------------------------------------------------------------*/
// Use audio terminals & types
//#define LONG_DESC

// Use alternate setting 0&1

//#define USE_AS1

#ifdef USE_AS1
#define LAS 1
#else
#define LAS 0
#endif

#ifdef LONG_DESC
#define LDL 1
#else
#define LDL 0
#endif

#define CONF_DESC_LEN (0x9)
//                     CF0 

#define IF0_CSPEC_LEN (LDL*(0x9+0xc+0x9+0x9+0x9+0x9))
//                          IF0 IT1 FU2 OT3 OT4 OT5

// IF0 len
#define IF0_DESC_LEN (0x9+IF0_CSPEC_LEN)
//                    IF0 

// IF1 len
#define IF1_ASDESC_LEN (LAS*0x9+0x9+LDL*(0x7+0xb)+0x7+LDL*0x7)
//                          AS0 AS1      A1G A1F   E1S    E1G

// IF2 len
#define IF2_DESC_LEN (0x9+0x7+0x7)
//                    IF2 EPI EPO

#define DESC_TOT_LEN (CONF_DESC_LEN+IF0_DESC_LEN+IF1_ASDESC_LEN+IF2_DESC_LEN)

static const short usb_config0_descriptor[] = 
{
  // Configuration descriptor for CF0

  0x09,                 // bLength 
  USB_DT_CONFIG,        // bDescriptorType 
  (DESC_TOT_LEN)&255, (DESC_TOT_LEN>>8)&255,     // wTotalLength     
  0x03,                 // bNumInterfaces 
  0x01,                 // bConfigurationValue 
  0x00,                 // iConfiguration (unused)
  0x00,                 // bmAttributs, not selfpowered 
  0x00,                 // MaxPower 

  // Interface 0 (audio control)

  // standard AC IF descriptor

  0x09,
  USB_DT_INTERFACE,
  0,                    // index IF
  0,                    // a index set
  0,                    // 0 EP
  USB_IFC_AUDIO,
  USB_IFSC_AUDIOCONTROL,
  0,
  0,

#ifdef LONG_DESC
  // class specific AC IF descriptor

  0x09,
  USB_CS_INTERFACE,
  USB_AC_HEADER,
  0x00,0x01,
  (IF0_CSPEC_LEN)&255,(IF0_CSPEC_LEN>>8)&255,
  1,                          // num stream IF
  1,                          // with IF 1

  // descriptor IT1

  0x0c,                       // len
  USB_CS_INTERFACE,
  USB_AC_INPUT_TERMINAL,
  0x01,                      // ID
  0x10,0x07,                 // FM rec
  0,                         // no assoc
  2,                         // 2 channels
  3,0,                       // L_front+R_front
  0,0,

  // descriptor FU2

  0x09,                       // len
  USB_CS_INTERFACE,
  USB_AC_FEATURE_UNIT,
  0x02,                      // ID
  0x01,                       // source
  0x8  ,                     // control size
  0x3,                       // mute+vol left
  0x3,                       // mute+vol right
   0,

  // descriptor OT3

  0x09,
  USB_CS_INTERFACE,
  USB_AC_OUTPUT_TERMINAL,
  3,
  0x02,0x03,                // headphone
  0,                        // no assoc
  2,                        // input  from FU2
  0,

  // descriptor OT4

  0x09,
  USB_CS_INTERFACE,
  USB_AC_OUTPUT_TERMINAL,
  4,
  0x03,0x06,                // line out
  0,                        // no assoc
  1,                        // input  from IT1
  0,

   // descriptor OT5

  0x09,
  USB_CS_INTERFACE,
  USB_AC_OUTPUT_TERMINAL,
  4,
  0x01,0x01,                // USB streaming
  0,                        // no assoc
  1,                        // input  from IT1
  0,

  // NO EP descriptor for IF 0
  // Interface 1 (I2S)
  // IF 1 AS 0
#endif
  // ----------------------------------------------------------------

 

  // ----------------------------------------------------------------

#ifdef USE_AS1
  0x09,
  USB_DT_INTERFACE,
  1,                       // if#
  0,                       // as#0
  0,                       // 0 EP
  USB_IFC_AUDIO,
  USB_IFSC_AUDIOSTREAMING,
  0,
  0,
#endif

  // IF 1 AS 1

  0x09,
  USB_DT_INTERFACE,
  1,                       // if#
  LAS,                     // as#1
  1,                       // 1 EP
  USB_IFC_AUDIO,
  USB_IFSC_AUDIOSTREAMING,
  0,
  0,

#ifdef LONG_DESC
  // AS 1 IF descr (general)

  0x07,
  USB_CS_INTERFACE,
  USB_AS_GENERAL,
  5,                      // Stream link to OT5
  1,                      // bdelay
  0x01,0x00,               // PCM

  // AS 1 IF descr (format)

  0x0b,
  USB_CS_INTERFACE,
  USB_AS_FORMAT_TYPE,
  USB_FT_FORMAT_TYPE_I,
  2,                      // 2 channels
  2,                      // 2bytes/slot
  16,                     // 16bit res
  1,                      // fixed freq
  0x80,0xbb,0x00,          // 48kHz
#endif
  // AS 1 EP descr (standard)

  0x07,
  USB_DT_ENDPOINT,
  0x88,                  // IN-8
  5,                     // async ISO
  0x90,0x01,            // >384bytes
  1,                     // interval 1
  
#ifdef LONG_DESC
  // AS 1 EP descr (specific)

  0x07,
  USB_CS_ENDPOINT,
  USB_EP_GENERAL,
  0x80,                  // max packets only?
  0,                     // no lock
  0,0,
#endif

  // Interface 2 (uC)
  // FIXME
 // ----------------------------------------------------------------
 // EP 2 for uC I/O
  
  0x09,
  USB_DT_INTERFACE,
  2,                    // index IF
  0,                    // a index set
  2,                    // 2 EPs
  0xff,
  0xff,
  0xff,
  0,
  
  0x07,
  USB_DT_ENDPOINT,
  0x82,                  // IN-2
  2,                     // bulk
  0x40,0x00,            // 64 bytes
  0,                     // no interval

  0x07,
  USB_DT_ENDPOINT,
  0x02,                  // OUT-2
  2,                     // bulk
  0x40,0x00,            // 64 bytes
  0  ,                   // no interval
};
