; ice.asm
; AN2131 Soft-In-Circuit-Emulator V0.1
; 26.10.00 GA
	.module ice
	.include "regezusb.asm"
	
	.area CODE (CODE)
	.globl ice_epout_isr
	.globl _init_debug
	.globl ice_breakpoint
	.globl ice_breakpoint_flag

; USB commands via OUT 1

ICE_GO		= 1
ICE_R_SFR 	= 2
ICE_W_SFR	= 3
ICE_R_MEM	= 4
ICE_W_MEM	= 5
ICE_R_IRAM	= 6
ICE_W_IRAM	= 7
ICE_RESET	= 0x10

ICEEP 		= 2
ICEVAL		= (1<<ICEEP)
OUTBUF 		= OUT2BUF
INBUF 		= IN2BUF
INBC		= IN2BC
OUTBC		= OUT2BC
OUTCS		= OUT2CS
INCS		= IN2CS
	
ice_fn_dispatch:
	mov dptr,#OUTA
	movx a,@dptr
	xrl a,#0x80
	movx @dptr,a

	mov dptr,#OUTCS
	movx a,@dptr
	anl a,#2
	cjne a,#0,ice_fn_dispatch	; wait for !busy

	mov dptr,#OUTBUF
	movx a,@dptr			; outbuf[0] cmd
	inc dptr
	cjne a,#ICE_GO,_l1
	ljmp _exit_dispatch
_l1:
	cjne a,#ICE_R_IRAM,_l2

; READ SFR (ADDR) -> (BYTE)
	movx a,@dptr
	mov r0,a
	mov a,@r0
	mov dptr,#INBUF
	movx @dptr,a
	sjmp _set_inbc
_l2:
	cjne a,#ICE_W_IRAM,_l3
; WRITE SFR (ADDR)
	movx a,@dptr		; outbuf[1] addr
	mov r0,a
	inc dptr		; outbuf[2] data
	movx a,@dptr
	mov @r0,a
	sjmp _set_inbc
_l3:
	cjne a,#ICE_R_MEM,_l4
; READ MEM (LOW HIGH) -> (2 Bytes)
	movx a,@dptr		; outbuf[1] addrL
	mov r0,a
	inc dptr
	movx a,@dptr		; outbuf[2] addrH
	mov DPL,r0
	mov DPH,a
	movx a,@dptr		; Byte 0
	mov r0,a
	inc dptr
	movx a,@dptr		; Byte 1

	mov dptr,#INBUF
	movx @dptr,a
	mov a,r0
	inc dptr
	movx @dptr,a
	sjmp _set_inbc
_l4:
	cjne a,#ICE_W_MEM,_l5
; WRITE MEM (LOW HIGH VALUE)
	movx a,@dptr		; outbuf[1] addrL
	mov r0,a
	inc dptr
	movx a,@dptr		; outbuf[2] addrH
	push acc
	inc dptr
	movx a,@dptr		; outbuf[3] data
	mov DPL,r0
	pop DPH			; addrH -> DPH
	movx @dptr,a		; write
	sjmp _set_inbc
_l5:
	cjne a,#ICE_RESET,_l6
; ICE_RESET
	mov dptr,#INCS
	mov a,#2;
	movx @dptr,a
	sjmp restart
_l6:
	cjne a,#ICE_R_SFR,_l7
	movx a,@dptr		; outbuf[1] addr
	mov dptr,#_mod1+1
	movx @dptr,a		; write reg;
_mod1:
	mov a,0			; 0 is modified!
	mov dptr,#INBUF
	movx @dptr,a
	sjmp _set_inbc
_l7:
	cjne a,#ICE_W_SFR,restart
	movx a,@dptr		; outbuf[1] addr
	mov dptr,#_mod2+1
	movx @dptr,a		; write reg;
	mov dptr,#OUTBUF+2
	movx a,@dptr		; outbuf[2] data
_mod2:
	mov 0,a			; 0 is modified!
;	ajmp _set_inbc
_set_inbc:

	mov dptr,#INBC
	mov a,#64
	movx @dptr,a

restart:
	lcall clr_cmd

	ljmp ice_fn_dispatch

_exit_dispatch:
clr_cmd:
	clr a
	mov dptr,#OUTBUF
	movx @dptr,a		; clr CMD
	mov dptr,#OUTBC
	movx @dptr,a		; rearm buf
	ret
;-----------------------------------------------------------------

ice_epout_isr:
	push DPS
	push acc
	push dpl
	push dph
	push psw
	mov  psw,#0x00 ; Bank 0
	push 0 ; R0

	lcall ice_fn_dispatch	

exit_ice:
	mov a,EXIF		; clr main irq
	clr acc.4
	mov EXIF,a

	mov dptr,#OUT07IRQ
	mov a,#ICEVAL		
	movx @dptr,a		; clr IRQ	

	mov dptr,#OUT07IEN
	movx a,@dptr
	orl a,#ICEVAL		; Enable USB IRQ
	movx @dptr,a
	
; restore regs
	pop 0
	pop psw
	pop dph
	pop dpl
	pop acc
	pop DPS
	reti

;-----------------------------------------------------------------
ice_breakpoint_flag:
	nop

ice_breakpoint:
	push DPS
	push acc
	push dpl
	push dph
	push psw
	mov  psw,#0x00 ; Bank 0
	push 0 ; R0

	mov dptr,#OUT07IEN ; disable USBIRQ
	movx a,@dptr
	anl a,#~ICEVAL
	movx @dptr,a

	mov dptr,#ice_breakpoint_flag
	mov a,#1
	movx @dptr,a

	mov dptr,#OUTA
	movx a,@dptr
	xrl a,#0x40
	movx @dptr,a

	lcall ice_fn_dispatch

	sjmp exit_ice
;----------------------------------------------------------
_init_debug:
	mov a,#ICEVAL;

	mov dptr,#IN07VAL
	movx @dptr,a 		; enable USB IN

	inc dptr		; OUT07VAL
	movx @dptr,a 		; enable USB OUT

	mov dptr,#OUT07IEN
	movx @dptr,a 		; enable USB OUT IRQ

	mov IE,#0x80 		; global IRQ enable

	mov a,#1

	mov dptr,#USBBAV	; enable autovector IRQ
	movx @dptr,a

	mov EIE,a 		; enable USB IRQs
	ret
